#!/usr/bin/env python
#coding: utf-8

import os
import ftplib
from StringIO import StringIO
import subprocess
import tempfile

CCRYPT = "/usr/bin/ccrypt"

FTP_HOST = "<host oder ip>"
FTP_USER = "<benutzername>"
FTP_PASS_FILE = os.path.join(os.path.dirname(os.path.abspath(__file__)), "ftppasswd")
FTP_PASS = file(FTP_PASS_FILE, "r").readline().strip()
CCRYPT_PASS_FILE = os.path.join(os.path.dirname(os.path.abspath(__file__)), "ccryptpasswd")
PLACEHOLDER = "_placeholder"

MYSQL_SOURCE_DIR = "/backup/mysql"
MYSQL_SOURCE_CURRENT_FILE = "/backup/mysql/.current"
MYSQL_DEST_DIR = "mysql"
MYSQL_DEST_CURRENT_FILE = "current"
MYSQL_DEST_MAX_BACKUPS = 10

FULL_SOURCE_DIR = "/backup/full"
FULL_SOURCE_CURRENT_FILE = "/backup/full/.current"
FULL_DEST_DIR = "full"
FULL_DEST_CURRENT_FILE = "current"
FULL_DEST_MAX_BACKUPS = 2


def copy_to_ftp(
    source_dir, source_current_file, dest_dir, dest_current_file, dest_max_backups
):
    # Verbinden
    ftp = ftplib.FTP(FTP_HOST, FTP_USER, FTP_PASS)
    
    # Platzhalter-Datei im Root-Ordner erstellen/ueberscheiben
    ftp.storbinary("STOR %s" % os.path.join("/", PLACEHOLDER), StringIO())

    # Basisordner erstellen
    if (
        os.path.join("/", dest_dir) not in
        (os.path.join("/", item) for item in ftp.nlst("/"))
    ):
        ftp.mkd(dest_dir)

    # Platzhalter-Datei im Basis-Ordner erstellen/ueberscheiben
    ftp.storbinary(
        "STOR %s" % os.path.join("/", dest_dir, PLACEHOLDER), 
        StringIO()
    )
    
    # QUELLE Current-Datei auslesen
    source_current = int(file(source_current_file, "r").readline().strip())
    
    # ZIEL Current-Datei auslesen
    dest_current_file_path = os.path.join("/", dest_dir, dest_current_file)
    if dest_current_file_path in ftp.nlst(os.path.join("/", dest_dir)):
        current_file = StringIO()
        ftp.retrbinary("RETR %s" % dest_current_file_path, current_file.write)
        current_file.seek(0)
        try:
            dest_current = int(current_file.readline().strip()) + 1
        except ValueError:
            dest_current = 1
    else:
        dest_current = 1
    if dest_current > dest_max_backups:
        dest_current = 1
    
    # Pfade herausfinden
    full_source_dir = os.path.join(source_dir, str(source_current))
    full_dest_dir = os.path.join("/", dest_dir, str(dest_current))
    
    # Zielordner löschen
    if full_dest_dir in ftp.nlst(os.path.join("/", dest_dir)):
        for full_filename in ftp.nlst(full_dest_dir):
            ftp.delete(full_filename)
        ftp.rmd(full_dest_dir)
       
    # Zielordner und Platzhalter-Datei erstellen
    ftp.mkd(full_dest_dir)
    ftp.storbinary(
        "STOR %s" % os.path.join(full_dest_dir, PLACEHOLDER), 
        StringIO()
    )
    
    # Dateien des Quellordner durchlaufen
    for filename in os.listdir(full_source_dir):
        # Verbindung schliessen
        try:
            ftp.quit()
            ftp.close()
        except: pass
        
        # Pfade
        full_source_filename = os.path.join(full_source_dir, filename)
        full_dest_filename = os.path.join(full_dest_dir, filename + ".cpt")

        print "Copy to FTP:", full_source_filename
        
        # Datei verschluesseln
        tmp_file = tempfile.TemporaryFile()
        args = [CCRYPT, "--encrypt", '--keyfile=%s' % CCRYPT_PASS_FILE]
        proc = subprocess.Popen(
            args, stdout = tmp_file, stdin = file(full_source_filename, "rb"),
            stderr = subprocess.PIPE
        )
        (stdout_str, stderr_str) = proc.communicate()
        if stderr_str:
            for line in stderr_str.splitlines():
                print "    ", line.rstrip()
                
        # Verbindung neu oeffnen
        ftp = ftplib.FTP(FTP_HOST, FTP_USER, FTP_PASS)

        # Datei in den Zielordner kopieren
        tmp_file.seek(0)
        ftp.storbinary("STOR %s" % full_dest_filename, tmp_file)
        tmp_file.close()
    
    # Ziel Current-Datei erstellen/ueberschreiben
    current_file = StringIO()
    current_file.write(str(dest_current))
    current_file.seek(0)
    ftp.storbinary("STOR %s" % dest_current_file_path, current_file)
    
    # Platzhalter-Dateien entfernen
    try:
        ftp.delete(os.path.join(full_dest_dir, PLACEHOLDER))
    except ftplib.error_perm:
        pass
    try:
        ftp.delete(os.path.join(dest_dir, PLACEHOLDER))
    except ftplib.error_perm:
        pass
    try:
        ftp.delete(os.path.join("/", PLACEHOLDER))
    except ftplib.error_perm:
        pass
    
    # Fertig
    try:
        ftp.quit()
        ftp.close()
    except:
        pass


def copy_to_ftp__mysql():
    copy_to_ftp(
        MYSQL_SOURCE_DIR, MYSQL_SOURCE_CURRENT_FILE, MYSQL_DEST_DIR,
        MYSQL_DEST_CURRENT_FILE, MYSQL_DEST_MAX_BACKUPS
    )


def copy_to_ftp__full():
    copy_to_ftp(
        FULL_SOURCE_DIR, FULL_SOURCE_CURRENT_FILE, FULL_DEST_DIR,
        FULL_DEST_CURRENT_FILE, FULL_DEST_MAX_BACKUPS
    )


def main():
    # Letzte MYSQL-Sicherung zum FTP-Server kopieren
    copy_to_ftp__mysql()

    # Letzte FULL-Sicherung zum FTP-Server kopieren
    copy_to_ftp__full()


if __name__ == "__main__":
    main()






    
